/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Preferences Toolbar 3.
 *
 * The Initial Developer of the Original Code is
 * Manuel Reimer.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): Aaron Andersen <aaron@xulplanet.com>
 *                 Kevin Teuscher
 *                 Manuel Reimer <Manuel.Reimer@gmx.de>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


// note: global vars in this file don't need "prefbar" prefix in their names

var prefbarActiveTree = null;
var allTree = null;
var enabledTree = null;

// some of the buttons in the middle of the panel
var editButton = null;
var deleteButton = null;
var exportButton = null;

var RDF = null;
var mDatasource = null;
var RDFCU = null;

function Startup() {
  setTimeout(DelayedStartup, 0);
}
function DelayedStartup() {
  allTree = document.getElementById("allTree");
  enabledTree = document.getElementById("enabledTree");

  allTree.addEventListener("click", TreeClick, true);
  enabledTree.addEventListener("click", TreeClick, true);

  goPrefBar.RDF.AddmDatasource(allTree);
  goPrefBar.RDF.AddmDatasource(enabledTree);

  RDF = goPrefBar.RDF.RDFService;
  mDatasource = goPrefBar.RDF.mDatasource;

  RDFCU = Components.classes['@mozilla.org/rdf/container-utils;1'].getService();
  RDFCU = RDFCU.QueryInterface(Components.interfaces.nsIRDFContainerUtils);

  editButton = document.getElementById("itemEdit");
  deleteButton = document.getElementById("itemDelete");
  exportButton = document.getElementById("itemExport");

  // Set the preset for the selections.
  enabledTree.focus();
  if (allTree.view) allTree.view.selection.clearSelection();
  if (enabledTree.view) enabledTree.view.selection.select(0);

  window.addEventListener("unload", prefbarWriteToDisk, true);
}

function TreeClick(event) {
  if (event.detail != 2) return; // doubleclick

  event.preventDefault();
  event.stopPropagation();
  prefbarItemEdit();
}

function TreeKeypress(event) {
  if (event.keyCode != 46) return;

  prefbarItemDelete();
}

function AllTreeFocus() {
  // Remove selection from other tree
  if (enabledTree.view) enabledTree.view.selection.clearSelection();
  prefbarActiveTree = allTree;
}
function EnabledTreeFocus() {
  // Remove selection from other tree
  if(allTree.view) allTree.view.selection.clearSelection();
  prefbarActiveTree = enabledTree;
}

function PopupShowing() {
  if (!prefbarActiveTree) return false;

  var selectionCount = prefbarActiveTree.view.selection.count;
  if (selectionCount == 0) return false;

  editButton.setAttribute("disabled", (selectionCount != 1));
  return true;
}


function ItemNew(aType) {
  if (!aType) return;
  var item1 = enabledTree.contentView.getItemAtIndex(0);
  window.openDialog("chrome://prefbar/content/newItem/newItem.xul", "newItemDialog", "chrome,titlebar,dialog,modal,resizable", aType);
  var item2 = enabledTree.contentView.getItemAtIndex(0);

  if (item1 != item2) {
    enabledTree.builder.rebuild();
    enabledTree.focus();
    setTimeout(function() {
      enabledTree.view.selection.select(0);
    });
  }
}

function prefbarItemEdit() {
  var selections = prefbarGetTreeSelections(prefbarActiveTree);
  if (selections.length != 1) return;
  var selItemId = selections[0];

  window.openDialog("chrome://prefbar/content/newItem/newItem.xul", "editItemDialog", "chrome,titlebar,dialog,modal,resizable", selItemId);
}


function prefbarItemDelete() {
  var selections = prefbarGetTreeSelections(prefbarActiveTree);
  var len = selections.length;
  if (len < 1) return;

  if(!goPrefBar.msgYesNo(window, goPrefBar.GetString("pref-editbar.properties", "questiondelete"))) return;

  var items = new Array();
  for(var i = 0; i < len; i++) {
    var selItemId = selections[i];
    var selItemCN = getCNameById(selItemId);
    items[selItemId] = selItemCN;
  }

  for (var delitem in items) {
    var Rdelitem = RDF.GetResource(delitem);
    var parent = items[delitem];
    var Rparent = RDF.GetResource(parent);
    var Cparent = GetSeq(Rparent);

    // If the item, to delete, is a container, then move all children to
    // the parent container, to empty the container.
    if (RDFCU.IsContainer(mDatasource, Rdelitem)) {
      var Cdelitem = GetSeq(Rdelitem);
      var destPos = Cparent.IndexOf(Rdelitem);
      var moveitems = Cdelitem.GetElements();

      while(moveitems.hasMoreElements()) {
        var Rmoveitem = moveitems.getNext();
        var moveitem = Rmoveitem.QueryInterface(Components.interfaces.nsIRDFResource).Value;

        if (moveitems.hasMoreElements())
          Cdelitem.RemoveElement(Rmoveitem, false);
        else
          Cdelitem.RemoveElement(Rmoveitem, true);

        Cparent.InsertElementAt(Rmoveitem, destPos, true);
        if (moveitem in items) items[moveitem] = parent;
        destPos++;
      }
    }

    var properties = mDatasource.ArcLabelsOut(Rdelitem);

    while(properties.hasMoreElements()) {
      var Rproperty = properties.getNext();

      var Lvalue = mDatasource.GetTarget(Rdelitem, Rproperty, true);
      mDatasource.Unassert(Rdelitem, Rproperty, Lvalue);
    }

    Cparent.RemoveElement(Rdelitem, true);
  }

  prefbarActiveTree.builder.rebuild();
}


function prefbarItemExport() {
  var nsIFilePicker = Components.interfaces.nsIFilePicker;
  var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);

  fp.init(window, "Export Buttons to File:", nsIFilePicker.modeSave);
  fp.appendFilter("ButtonCollections", "*.btn");
    //fp.appendFilters(nsIFilePicker.filterAll);
  fp.defaultExtension = "btn";

  var res=fp.show();
  if (res==nsIFilePicker.returnOK || res == nsIFilePicker.returnReplace) {
    var selections = prefbarGetTreeSelections(prefbarActiveTree);
    var toexport = new Array;
    for (var i = 0; i <= selections.length - 1; i++) {
      var itemId = selections[i];

      toexport.push(itemId);
    }

    // Call the exporter
    goPrefBar.ImpExp.Export(window, toexport, fp.fileURL.spec);

    delete toexport;
  }
}


function prefbarItemImport() {
  var nsIFilePicker = Components.interfaces.nsIFilePicker;
  var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);

  fp.init(window, "Import Buttons from File:", nsIFilePicker.modeOpen);
  fp.appendFilter("ButtonCollections", "*.btn");
  //fp.appendFilters(nsIFilePicker.filterAll);
  fp.defaultExtension = "btn";

  var res=fp.show();
  if (res==nsIFilePicker.returnOK) {
    goPrefBar.ImpExp.Import(window, fp.fileURL.spec);
  }
}

const gDragFlavor = "text/x-moz-prefbar-button";
var gDragArray;

function BeginDragTree(event, tree) {
  goPrefBar.dump("BeginDragTree");

  if (event.originalTarget.localName != "treechildren")
    return false;

  var row = tree.treeBoxObject.getRowAt(event.clientX, event.clientY);
  goPrefBar.dump("BeginDragTree: row: " + row);
  if (row == -1)
    return false;

  gDragArray = new Array();

  var selection = prefbarGetTreeSelections(tree);
  for (var i = 0; i < selection.length; ++i ) {
    var curItem = selection[i];
    var ItemArray = new Array(curItem, getCNameById(curItem));
    gDragArray.push(ItemArray);
  }

  // We just drag the word "prefbar" the usual way.
  // The real data is in gDragArray.
  event.dataTransfer.effectAllowed = "move";
  event.dataTransfer.setData(gDragFlavor, "prefbar");

  return false;  // don't propagate the event if a drag has begun
}

function DragOverTree(event) {
  if (event.dataTransfer.types.contains(gDragFlavor)) {
    event.preventDefault();
    return false;
  }

  return true;
}

function DropOnTree(event, tree) {
  goPrefBar.dump("DropOnTree");

  // The drop procedure may take really long (depending on how much items
  // have to be moved)...
  window.setCursor("wait");
  saveOpenStates();

  var idPlaceBefore = null;
  var idPlaceInside = null;
  var PlaceAfter = false;

  var tbo = tree.treeBoxObject;
  if (tbo.treeBody && tbo.treeBody.childNodes.length > 0) {
    var row = tbo.getRowAt(event.clientX, event.clientY);
    goPrefBar.dump("DropOnTree: Dropped On Row: " + row);
    if (row != -1) {
      var rowHeight = tbo.rowHeight;
      var rowY = tbo.treeBody.boxObject.y;
      var firstRow = tbo.getFirstVisibleRow();

      var ItemPos = rowY + rowHeight * (row - firstRow);
      var pastePos = (event.clientY - ItemPos) / (rowHeight / 4);

      idPlaceBefore = tree.contentView.getItemAtIndex(row).getAttribute("id");

      var item = document.getElementById(idPlaceBefore);
      if (item.getAttribute("container") == "true") {
        if (item.getAttribute("empty") == "true") {
          if (pastePos > 1 && pastePos < 3) {
            idPlaceInside = idPlaceBefore;
            idPlaceBefore = null;
          }
          else if (pastePos >= 3)
            PlaceAfter = true;
        }
        else if (item.getAttribute("open") == "true") {
          if (pastePos > 2) {
            idPlaceInside = idPlaceBefore;
            idPlaceBefore = null;
          }
        }
        else if (pastePos > 2)
          PlaceAfter = true;
      }
      else if (pastePos > 2)
        PlaceAfter = true;
    }
    else {
      goPrefBar.dump("DropOnTree: Dropped after last item");
      var lastRow = tbo.treeBody.childNodes.length - 1;
      idPlaceBefore = tbo.treeBody.childNodes[lastRow].getAttribute("id");
      PlaceAfter = true;
    }
  }
  else {
    goPrefBar.dump("DropOnTree: Dropped On Empty Tree");
  }

  // Get target sequence and container
  var targetseq;
  if (idPlaceBefore)
    targetseq = getCNameById(idPlaceBefore);
  else if (idPlaceInside) {
    targetseq = idPlaceInside;
    savedOpenState[idPlaceInside] = "true";
  }
  else
    targetseq = tree.ref;
  var targetcontainer = GetSeq(RDF.GetResource(targetseq));

  // Containers may only be moved as one single item and they may only be
  // moved to the enabled root level. Otherwise delete all containers
  // from gDragArray
  if (gDragArray.length > 1 ||
      targetseq != "urn:prefbar:browserbuttons:enabled") {
    for (var i = gDragArray.length - 1; i >= 0; i--) {
      RcurId = RDF.GetResource(gDragArray[i][0]);
      if (RDFCU.IsContainer(mDatasource, RcurId))
        gDragArray.splice(i, 1);
    }
  }

  // first run: Kill all items from old seq
  var dragcnt = gDragArray.length;
  for (var i = 0; i < dragcnt; i++) {
    var curitem = gDragArray[i];
    var RcurId = RDF.GetResource(curitem[0]);
    var RcurCN = RDF.GetResource(curitem[1]);
    var container = RDFCU.MakeSeq(mDatasource, RcurCN);
    container.RemoveElement(RcurId, true);
  }

  // Now get target position.
  var newpos;
  if (!idPlaceBefore)
    newpos = 1;
  else {
    newpos = targetcontainer.IndexOf(RDF.GetResource(idPlaceBefore));
    if (newpos == -1)
      newpos = 1;
    else
      if (PlaceAfter) newpos++;
  }

  // Second run: Add all items to new seq on their new position
  for (var i = 0; i < dragcnt; i++) {
    var curitem = gDragArray[i];
    var RcurId = RDF.GetResource(curitem[0]);
    targetcontainer.InsertElementAt(RDF.GetResource(curitem[0]), newpos, true);
    newpos++;
  }

  // Focus target tree
  tree.builder.rebuild();
  tree.focus();
  restoreOpenStates();

  // Add selection for newly added items
  if (dragcnt > 0) {
    for (var index = 0; index < tree.view.rowCount; index++) {
      if (tree.contentView.getItemAtIndex(index).id == gDragArray[0][0]) {
        tree.view.selection.rangedSelect(index, index + gDragArray.length - 1, false);
        break;
      }
    }
  }

  window.setCursor("auto");
  return true;
}


function getCNameById(id) {
  var item = document.getElementById(id);
  item = item.parentNode.parentNode;
  if (item.localName == "treeitem")
    return item.id;
  else
    return item.ref;
}

var savedOpenState = new Array();
function saveOpenStates() {
  var tb = enabledTree.treeBoxObject.treeBody;
  var len = tb.childNodes.length;
  for (var index = 0; index < len; index++) {
    var curnode = tb.childNodes[index];
    if (curnode.getAttribute("container") == "true")
      savedOpenState[curnode.id] = curnode.getAttribute("open");
  }
}
function restoreOpenStates() {
  for (var index in savedOpenState) {
    document.getElementById(index).setAttribute("open", savedOpenState[index]);
  }
}

function GetSeq(RSeq) {
  if (!RDFCU.IsContainer(mDatasource, RSeq)) return false;
  return RDFCU.MakeSeq(mDatasource, RSeq);
}

function prefbarWriteToDisk() {
  goPrefBar.RDF.mDatasource.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource).Flush();
}

function prefbarGetTreeSelections(tree) {
  var selections = [];
  var select = tree.view.selection;
  if (select) {
    var count = select.getRangeCount();
    var min = new Object();
    var max = new Object();
    for (var i = 0; i < count; i++) {
      select.getRangeAt(i, min, max);
      for (var k=min.value; k<=max.value; k++) {
        if (k != -1) {
          selections[selections.length] = tree.contentView.getItemAtIndex(k).getAttribute("id");
        }
      }
    }
  }
  return selections;
}
